//package org.springblade.yeePay.controller;
//
//
//import io.swagger.annotations.Api;
//import io.swagger.annotations.ApiOperation;
//import io.swagger.annotations.ApiParam;
//import liquibase.pro.packaged.C;
//import org.springblade.core.tool.api.R;
//import org.springblade.yeePay.entity.*;
//import org.springblade.yeePay.service.YeePayService;
//import org.springframework.beans.factory.annotation.Autowired;
//import org.springframework.validation.annotation.Validated;
//import org.springframework.web.bind.annotation.*;
//
//import javax.validation.Valid;
//import java.util.Map;
//
//@RestController
//@RequestMapping("/yeePay")
//@Api(tags = "易宝支付模块")
//@Validated
//public class YeePayController {
//
//	@Autowired
//	YeePayService yeePayService;
//
//	/**
//	 * 统一下单接口
//	 * @param initOrderDto 下单信息
//	 */
//	@PostMapping("initOrder")
//	@ApiOperation("统一下单接口")
//	public R initOrder(@RequestBody @Valid InitOrderDto initOrderDto){
//		return yeePayService.initOrder(initOrderDto);
//	}
//
//	/**
//	 * 公众号配置接口
//	 * @param ConfigAppDto 公众号配置信息
//	 */
//	@PostMapping("configApp")
//	@ApiOperation("公众号配置接口")
//	public R configApp(@RequestBody @Valid ConfigAppDto ConfigAppDto){
//		return yeePayService.configApp(ConfigAppDto);
//	}
//
//	/**
//	 * 查询公众号配置接口
//	 */
//	@GetMapping("queryApp")
//	@ApiOperation("查询公众号配置接口")
//	public R configApp(@RequestParam(required = false) String appIdType){
//		return yeePayService.queryApp(appIdType);
//	}
//
//	/**
//	 *	代付：转账到银行卡(单笔出款)
//	 * @param transferDto 转账信息
//	 * */
//	@PostMapping("transferToCard")
//	@ApiOperation("代付：转账到银行卡(单笔出款)")
//	public R transferToCard(@RequestBody @Valid TransferDto transferDto){
//		return yeePayService.transferToCard(transferDto);
//	}
//
//	/**
//	 *	代付：转账到银行卡(批量出款)
//	 * @param batchTransferDto 转账信息
//	 * */
//	@PostMapping("batchTransfer")
//	@ApiOperation("代付：转账到银行卡(批量出款)")
//	public R batchTransfer(@RequestBody @Valid BatchTransferDto batchTransferDto){
//		return yeePayService.batchTransferToCard(batchTransferDto);
//	}
//
//
//	/**
//	 * 	转账订单查询
//	 * @param requestNo 易宝订单号
//	 * */
//	@GetMapping("queryTransferOrder")
//	@ApiOperation("转账订单查询")
//	public R queryTransferOrder(@RequestParam @ApiParam(value = "请求订单号",required = true) String requestNo ){
//		return yeePayService.queryTransferOrder(requestNo);
//	}
//
//	/**
//	 * 支付回调接口（支付结果通知）
//	 */
//	@PostMapping("notify")
//	@ApiOperation("支付回调接口（支付结果通知）")
//	public R orderNotify(@RequestParam(required = false) Map<String,Object> params){
//		yeePayService.orderNotify(params);
//		return R.success("操作成功！");
//	}
//
//	/**
//	 * 转账回调接口（转账结果通知）
//	 */
//	@PostMapping("/transfer/notify")
//	@ApiOperation("转账回调接口（转账结果通知）")
//	public R transferNotify(@RequestParam(required = false) Map<String,Object> params){
//		yeePayService.transferNotify(params);
//		return R.success("操作成功！");
//	}
//
//
//	/**
//	 * 转账回调接口（转账结果通知 批量）
//	 */
//	@PostMapping("/batchTransfer/notify")
//	@ApiOperation("转账回调接口（转账结果通知 批量）")
//	public R batchTransferNotify(@RequestParam(required = false) Map<String,Object> params){
//		yeePayService.batchTransferNotify(params);
//		return R.success("操作成功！");
//	}
//
//	/**
//	 * 账户余额查询
//	 */
//	@GetMapping("queryBalance")
//	@ApiOperation("账户余额查询")
//	public R queryBalance(){
//		return yeePayService.queryBalance();
//	}
//
//
//	/**
//	 * 网关充值回调接口（充值结果通知）
//	 */
//	@PostMapping("/gateway/notify")
//	@ApiOperation("网关充值回调接口（充值结果通知）")
//	public R gatewayNotify(@RequestParam(required = false) Map<String,Object> params){
//		yeePayService.gatewayNotify(params);
//		return R.success("操作成功！");
//	}
//
//	/**
//	 *	网关：充值
//	 * @param payGateWayDto 充值信息
//	 * */
//	@PostMapping("payByGateway")
//	@ApiOperation("网关：充值")
//	public R payByGateway(@RequestBody @Valid PayGateWayDto payGateWayDto){
//		return yeePayService.payByGateWay(payGateWayDto);
//	}
//
//	/**
//	 *	生成订单支付聚合码
//	 * @param createCodeDto 订单信息
//	 * */
//	@PostMapping("createCode")
//	@ApiOperation("生成订单支付聚合码")
//	public R createCode(@RequestBody @Valid CreateCodeDto createCodeDto){
//		return yeePayService.createCode(createCodeDto);
//	}
//
//	/**
//	 *	扫码支付
//	 * @param chargeDto 订单信息
//	 * */
//	@PostMapping("chargePay")
//	@ApiOperation("扫码支付")
//	public R chargePay(@RequestBody @Valid ChargeDto chargeDto){
//		return yeePayService.chargePay(chargeDto);
//	}
//}
